@@ -61,6 +61,7 @@ static int rows, cols;		/* window size of the curr window */
 int visual_bell = 0;
 int use_hardstatus = 1;		/* display status line in hs */
 char *printcmd = 0;
+int use_altscreen = 0;		/* enable alternate screen support? */
 
 unsigned char *blank;		/* line filled with spaces */
 unsigned char *null;		/* line filled with '\0' */
@@ -669,8 +670,19 @@ register int len;
 #ifdef FONT
 # ifdef DW_CHARS
 	  if (!curr->w_mbcs)
+	    {
+# endif
+	      if (c < 0x80 || curr->w_gr == 0)
+	        curr->w_rend.font = curr->w_FontL;
+# ifdef ENCODINGS
+	      else if (curr->w_gr == 2 && !curr->w_ss)
+	        curr->w_rend.font = curr->w_FontE;
+# endif
+	      else
+	        curr->w_rend.font = curr->w_FontR;
+# ifdef DW_CHARS
+	    }
 # endif
-	    curr->w_rend.font = (c >= 0x80 ? curr->w_FontR : curr->w_FontL);
 # ifdef UTF8
 	  if (curr->w_encoding == UTF8)
 	    curr->w_rend.font = 0;
@@ -1334,6 +1346,21 @@ int c, intermediate;
 	 /* case 40:	         132 col enable */
 	 /* case 42:	   NRCM: 7bit NRC character mode */
 	 /* case 44:	         margin bell enable */
+	    case 47:    /*       xterm-like alternate screen */
+	    case 1047:  /*       xterm-like alternate screen */
+	    case 1049:  /*       xterm-like alternate screen */
+	      if (use_altscreen)
+		{
+		  if (i)
+		    EnterAltScreen(curr);
+		  else
+		    LeaveAltScreen(curr);
+		  if (a1 == 47 && !i)
+		    curr->w_saved = 0;
+		  LRefreshAll(&curr->w_layer, 0);
+		  LGotoPos(&curr->w_layer, curr->w_x, curr->w_y);
+		}
+	      break;
 	 /* case 66:	   NKM:  Numeric keypad appl mode */
 	 /* case 68:	   KBUM: Keyboard usage mode (data process) */
 	    case 1000:	/* VT200 mouse tracking */
