@@ -1801,7 +1801,8 @@ void
 SetFont(new)
 int new;
 {
-  if (!display || D_rend.font == new)
+  int old = D_rend.font;
+  if (!display || old == new)
     return;
   D_rend.font = new;
 #ifdef ENCODINGS
@@ -1819,7 +1820,11 @@ int new;
     }
 
   if (!D_CG0 && new != '0')
-    new = ASCII;
+    {
+      new = ASCII;
+      if (old == new)
+	return;
+    }
 
   if (new == ASCII)
     AddCStr(D_CE0);
@@ -2347,7 +2352,7 @@ RefreshHStatus()
   evdeq(&D_hstatusev);
   if (D_status == STATUS_ON_HS)
     return;
-  buf = MakeWinMsgEv(hstatusstring, D_fore, '%', (D_HS && D_has_hstatus == HSTATUS_HS) ? D_WS : D_width - !D_CLP, &D_hstatusev);
+  buf = MakeWinMsgEv(hstatusstring, D_fore, '%', (D_HS && D_has_hstatus == HSTATUS_HS && D_WS > 0) ? D_WS : D_width - !D_CLP, &D_hstatusev);
   if (buf && *buf)
     {
       ShowHStatus(buf);
@@ -2847,7 +2852,7 @@ int ins;
     ChangeScrollRegion(ys, ye);		/* remove unusable region */
   if (D_x != D_width || D_y != y)
     {
-      if (D_CLP)			/* don't even try if !LP */
+      if (D_CLP && y >= 0)		/* don't even try if !LP */
         RefreshLine(y, D_width - 1, D_width - 1, 0);
       debug2("- refresh last char -> x,y now %d,%d\n", D_x, D_y);
       if (D_x != D_width || D_y != y)	/* sorry, no bonus */
@@ -2919,6 +2924,8 @@ int newtop, newbot;
 {
   if (display == 0)
     return;
+  if (newtop == newbot)
+    return;			/* xterm etc can't do it */
   if (newtop == -1)
     newtop = 0;
   if (newbot == -1)
@@ -3357,8 +3364,9 @@ char *data;
 #ifdef ENCODINGS
   if (D_encoding != (D_forecv ? D_forecv->c_layer->l_encoding : 0))
     {
-      int i, j, c;
+      int i, j, c, enc;
       char buf2[IOSIZE * 2 + 10];
+      enc = D_forecv ? D_forecv->c_layer->l_encoding : 0;
       for (i = j = 0; i < size; i++)
 	{
 	  c = ((unsigned char *)buf)[i];
@@ -3370,11 +3378,11 @@ char *data;
 	  if (pastefont)
 	    {
 	      int font = 0;
-	      j += EncodeChar(buf2 + j, c, D_forecv->c_layer->l_encoding, &font);
-	      j += EncodeChar(buf2 + j, 0, D_forecv->c_layer->l_encoding, &font);
+	      j += EncodeChar(buf2 + j, c, enc, &font);
+	      j += EncodeChar(buf2 + j, -1, enc, &font);
 	    }
 	  else
-	    j += EncodeChar(buf2 + j, c, D_forecv->c_layer->l_encoding, 0);
+	    j += EncodeChar(buf2 + j, c, enc, 0);
 	  if (j > sizeof(buf2) - 10)	/* just in case... */
 	    break;
 	}
