@@ -39,6 +39,14 @@ extern struct layer *flayer;
 extern char *screenencodings;
 
 static int  encmatch __P((char *, char *));
+# ifdef UTF8
+static int   recode_char __P((int, int, int));
+static int   recode_char_to_encoding __P((int, int));
+#  ifdef DW_CHARS
+static int   recode_char_dw __P((int, int *, int, int));
+static int   recode_char_dw_to_encoding __P((int, int *, int));
+#  endif
+# endif
 
 struct encoding {
   char *name;
@@ -293,21 +301,21 @@ InitBuiltinTabs()
     }
 }
 
-int
+static int
 recode_char(c, to_utf, font)
 int c, to_utf, font;
 {
   int f;
   unsigned short (*p)[2];
 
-  if (c < 256)
-    return c;
   if (to_utf)
     {
+      if (c < 256)
+	return c;
       f = (c >> 8) & 0xff;
       c &= 0xff;
       /* map aliases to keep the table small */
-      switch (c >> 8)
+      switch (f)
 	{
 	  case 'C':
 	    f ^= ('C' ^ '5');
@@ -339,20 +347,24 @@ int c, to_utf, font;
     }
   if (font == -1)
     {
-      for (font = 32; font < 256; font++)
+      if (c < 256)
+	return c;	/* latin1 */
+      for (font = 32; font < 128; font++)
 	{
 	  p = recodetabs[font].tab;
 	  if (p)
 	    for (; (*p)[1]; p++)
 	      {
 		if ((p[0][0] & 0x8000) && c <= p[0][1] && c >= p[-1][1])
-		  return c - p[-1][1] + p[-1][0];
+		  return (c - p[-1][1] + p[-1][0]) | (font << 8);
 	        if ((*p)[1] == c)
-		  return (*p)[0];
+		  return (*p)[0] | (font << 8);
 	      }
 	}
       return '?';
     }
+  if (c < 128 && (font & 128) != 0)
+    return c;
   if (font >= 32)
     {
       p = recodetabs[font].tab;
@@ -363,15 +375,19 @@ int c, to_utf, font;
 	}
       if (p)
 	for (; (*p)[1]; p++)
-	  if ((*p)[1] == c)
-	    return (*p)[0];
+	  {
+	    if ((p[0][0] & 0x8000) && c <= p[0][1] && c >= p[-1][1])
+	      return (c - p[-1][1] + p[-1][0]) | (font & 128 ? 0 : font << 8);
+	    if ((*p)[1] == c)
+	      return (*p)[0] | (font & 128 ? 0 : font << 8);
+	  }
     }
   return -1;
 }
 
 
 #ifdef DW_CHARS
-int
+static int
 recode_char_dw(c, c2p, to_utf, font)
 int c, *c2p, to_utf, font;
 {
@@ -437,7 +453,7 @@ int c, *c2p, to_utf, font;
 }
 #endif
 
-int
+static int
 recode_char_to_encoding(c, encoding)
 int c, encoding;
 {
@@ -457,7 +473,7 @@ int c, encoding;
 }
 
 #ifdef DW_CHARS
-int
+static int
 recode_char_dw_to_encoding(c, c2p, encoding)
 int c, *c2p, encoding;
 {
@@ -923,7 +939,12 @@ struct win *p;
   LoadFontTranslationsForEncoding(encoding);
 #endif
   if (encodings[encoding].usegr)
-    p->w_gr = 1;
+    {
+      p->w_gr = 2;
+      p->w_FontE = encodings[encoding].charsets[1];
+    }
+  else
+    p->w_FontE = 0;
   if (encodings[encoding].noc1)
     p->w_c1 = 0;
 }
@@ -1032,7 +1053,7 @@ int *fontp;
   int t, f, l;
 
   debug2("Encoding char %02x for encoding %d\n", c, encoding);
-  if (c == 0 && fontp)
+  if (c == -1 && fontp)
     {
       if (*fontp == 0)
 	return 0;
@@ -1067,7 +1088,7 @@ int *fontp;
         }
       return ToUtf8(bp, c);
     }
-  if ((c & 0xff00) && f == 0)
+  if ((c & 0xff00) && f == 0)	/* is_utf8? */
     {
 # ifdef DW_CHARS
       if (utf8_isdouble(c))
@@ -1297,7 +1318,7 @@ unsigned char *tbuf;
 	continue;
       j += EncodeChar(tbuf ? (char *)tbuf + j : 0, c, tenc, &font);
     }
-  j += EncodeChar(tbuf ? (char *)tbuf + j : 0, 0, tenc, &font);
+  j += EncodeChar(tbuf ? (char *)tbuf + j : 0, -1, tenc, &font);
   return j;
 }
 
@@ -1456,7 +1477,7 @@ int encoding;
 int *fontp;
 {
   int f, l;
-  f = c >> 16;
+  f = (c == -1) ? 0 : c >> 16;
   l = 0;
   if (fontp && f != *fontp)
     {
@@ -1486,7 +1507,7 @@ int *fontp;
 	  l += 3;
 	}
     }
-  if (c == 0)
+  if (c == -1)
     return l;
   if (c & 0xff00)
     {
