@@ -303,6 +303,9 @@ extern char *xrealloc __P((char *, int));
 extern void  ResizeLayersToCanvases __P((void));
 extern void  ResizeLayer __P((struct layer *, int, int, struct display *));
 extern int   MayResizeLayer __P((struct layer *));
+extern void  FreeAltScreen __P((struct win *));
+extern void  EnterAltScreen __P((struct win *));
+extern void  LeaveAltScreen __P((struct win *));
 
 /* sched.c */
 extern void  evenq __P((struct event *));
@@ -448,12 +451,6 @@ extern char *DoNLS __P((char *));
 #ifdef ENCODINGS
 # ifdef UTF8
 extern void  InitBuiltinTabs __P((void));
-extern int   recode_char __P((int, int, int));
-extern int   recode_char_to_encoding __P((int, int));
-#  ifdef DW_CHARS
-extern int   recode_char_dw __P((int, int *, int, int));
-extern int   recode_char_dw_to_encoding __P((int, int *, int));
-#  endif
 extern struct mchar *recode_mchar __P((struct mchar *, int, int));
 extern struct mline *recode_mline __P((struct mline *, int, int, int));
 extern int   FromUtf8 __P((int, int *));
