@@ -220,6 +220,7 @@ int nethackflag = 0;
 #ifdef MAPKEYS
 int maptimeout = 300000;
 #endif
+int maxwin = MAXWIN;
 
 
 struct layer *flayer;
@@ -713,6 +714,23 @@ char **av;
       else
 	break;
     }
+
+  real_uid = getuid();
+  real_gid = getgid();
+  eff_uid = geteuid();
+  eff_gid = getegid();
+  if (eff_uid != real_uid)
+    {		
+      /* if running with s-bit, we must install a special signal
+       * handler routine that resets the s-bit, so that we get a
+       * core file anyway.
+       */
+#ifdef SIGBUS /* OOPS, linux has no bus errors! */
+      signal(SIGBUS, CoreDump);
+#endif /* SIGBUS */
+      signal(SIGSEGV, CoreDump);
+    }
+
 #ifdef USE_LOCALE
   setlocale(LC_ALL, "");
 #endif
@@ -749,21 +767,6 @@ char **av;
 #endif
   if (ac)
     nwin.args = av;
-  real_uid = getuid();
-  real_gid = getgid();
-  eff_uid = geteuid();
-  eff_gid = getegid();
-  if (eff_uid != real_uid)
-    {		
-      /* if running with s-bit, we must install a special signal
-       * handler routine that resets the s-bit, so that we get a
-       * core file anyway.
-       */
-#ifdef SIGBUS /* OOPS, linux has no bus errors! */
-      signal(SIGBUS, CoreDump);
-#endif /* SIGBUS */
-      signal(SIGSEGV, CoreDump);
-    }
 
   /* make the write() calls return -1 on all errors */
 #ifdef SIGXFSZ
@@ -888,7 +891,7 @@ char **av;
 #ifdef MULTIUSER
       tty_mode = (int)st.st_mode & 0777;
 #endif
-      if ((n = secopen(attach_tty, O_RDWR, 0)) < 0)
+      if ((n = secopen(attach_tty, O_RDWR | O_NONBLOCK, 0)) < 0)
 	Panic(0, "Cannot open your terminal '%s' - please check.", attach_tty);
       close(n);
       debug1("attach_tty is %s\n", attach_tty);
@@ -1014,6 +1017,8 @@ char **av;
     }
   if ((st.st_mode & 0777) != 0700)
     Panic(0, "Directory %s must have mode 700.", SockPath);
+  if (SockMatch && index(SockMatch, '/'))
+    Panic(0, "Bad session name '%s'", SockMatch);
   SockName = SockPath + strlen(SockPath) + 1;
   *SockName = 0;
   (void) umask(oumask);
