@@ -50,7 +50,7 @@ extern int SilenceWait;
 extern int real_uid, real_gid, eff_uid, eff_gid;
 extern char Termcap[];
 extern char **NewEnv;
-extern int visual_bell;
+extern int visual_bell, maxwin;
 extern struct event logflushev;
 extern int log_flush, logtstamp_after;
 extern int ZombieKey_destroy, ZombieKey_resurrect;
@@ -531,7 +531,7 @@ struct NewWindow *newwin;
   register int n, i;
   int f = -1;
   struct NewWindow nwin;
-  int type;
+  int type, startat;
   char *TtyName;
 #ifdef MULTIUSER
   extern struct acluser *users;
@@ -547,16 +547,17 @@ struct NewWindow *newwin;
   debug1("NWin: wlock   %d\n", nwin.wlock);
   debug1("NWin: Lflag   %d\n", nwin.Lflag);
 
-  pp = wtab + nwin.StartAt;
+  startat = nwin.StartAt < maxwin ? nwin.StartAt : 0;
+  pp = wtab + startat;
 
   do
     {
       if (*pp == 0)
 	break;
-      if (++pp == wtab + MAXWIN)
+      if (++pp == wtab + maxwin)
 	pp = wtab;
     }
-  while (pp != wtab + nwin.StartAt);
+  while (pp != wtab + startat);
   if (*pp)
     {
       Msg(0, "No more windows.");
@@ -972,6 +973,8 @@ struct win *wp;
       RethinkViewportOffsets(cv);
     }
   wp->w_layer.l_cvlist = 0;
+  if (flayer == &wp->w_layer)
+    flayer = 0;
 
 #ifdef MULTIUSER
   FreeWindowAcl(wp);
